// src/utilities/index.ts
var tryParseJSON = (jsonString) => {
  try {
    const json = JSON.parse(jsonString);
    return json;
  } catch (e) {
    return jsonString;
  }
};
var generateFormData = (formData, preserveStringified = false) => {
  const outputObject = {};
  for (const [key, value] of formData.entries()) {
    const data = preserveStringified ? value : tryParseJSON(value.toString());
    const keyParts = key.split(".");
    let currentObject = outputObject;
    for (let i = 0; i < keyParts.length - 1; i++) {
      const keyPart = keyParts[i];
      if (!currentObject[keyPart]) {
        currentObject[keyPart] = /^\d+$/.test(keyParts[i + 1]) ? [] : {};
      }
      currentObject = currentObject[keyPart];
    }
    const lastKeyPart = keyParts[keyParts.length - 1];
    const lastKeyPartIsArray = /\[\d*\]$|\[\]$/.test(lastKeyPart);
    if (lastKeyPartIsArray) {
      const key2 = lastKeyPart.replace(/\[\d*\]$|\[\]$/, "");
      if (!currentObject[key2]) {
        currentObject[key2] = [];
      }
      currentObject[key2].push(data);
    }
    if (!lastKeyPartIsArray) {
      if (/^\d+$/.test(lastKeyPart)) {
        currentObject.push(data);
      } else {
        currentObject[lastKeyPart] = data;
      }
    }
  }
  return outputObject;
};
var getFormDataFromSearchParams = (request, preserveStringified = false) => {
  const searchParams = new URL(request.url).searchParams;
  return generateFormData(searchParams, preserveStringified);
};
var isGet = (request) => request.method === "GET" || request.method === "get";
var getValidatedFormData = async (request, resolver) => {
  const data = isGet(request) ? getFormDataFromSearchParams(request) : await parseFormData(request);
  const validatedOutput = await validateFormData(data, resolver);
  return { ...validatedOutput, receivedValues: data };
};
var validateFormData = async (data, resolver) => {
  const dataToValidate = data instanceof FormData ? Object.fromEntries(data) : data;
  const { errors, values } = await resolver(
    dataToValidate,
    {},
    { shouldUseNativeValidation: false, fields: {} }
  );
  if (Object.keys(errors).length > 0) {
    return { errors, data: void 0 };
  }
  return { errors: void 0, data: values };
};
var createFormData = (data, stringifyAll = true) => {
  const formData = new FormData();
  if (!data) {
    return formData;
  }
  Object.entries(data).map(([key, value]) => {
    if (value instanceof FileList) {
      for (let i = 0; i < value.length; i++) {
        formData.append(key, value[i]);
      }
      return;
    }
    if (value instanceof File || value instanceof Blob) {
      formData.append(key, value);
    } else {
      if (stringifyAll) {
        formData.append(key, JSON.stringify(value));
      } else {
        formData.append(
          key,
          typeof value === "string" ? value : JSON.stringify(value)
        );
      }
    }
  });
  return formData;
};
var parseFormData = async (request, preserveStringified = false) => {
  const formData = request instanceof Request ? await request.formData() : request;
  return generateFormData(formData, preserveStringified);
};
var mergeErrors = (frontendErrors, backendErrors, validKeys = [], depth = 0) => {
  if (!backendErrors) {
    return frontendErrors;
  }
  for (const [key, rightValue] of Object.entries(backendErrors)) {
    if (!validKeys.includes(key.toString()) && validKeys.length && depth === 0) {
      continue;
    }
    if (typeof rightValue === "object" && !Array.isArray(rightValue)) {
      if (!frontendErrors[key]) {
        frontendErrors[key] = {};
      }
      mergeErrors(frontendErrors[key], rightValue, validKeys, depth + 1);
    } else if (rightValue) {
      frontendErrors[key] = rightValue;
    }
  }
  return frontendErrors;
};

// src/hook/index.tsx
import React from "react";
import {
  useActionData,
  useSubmit,
  useNavigation
} from "@remix-run/react";
import {
  useFormContext
} from "react-hook-form";
import { useForm, FormProvider } from "react-hook-form";
var useRemixForm = ({
  submitHandlers,
  submitConfig,
  submitData,
  fetcher,
  stringifyAllValues = true,
  ...formProps
}) => {
  var _a, _b, _c, _d;
  const [isSubmittedSuccessfully, setIsSubmittedSuccessfully] = React.useState(false);
  const actionSubmit = useSubmit();
  const actionData = useActionData();
  const submit = (_a = fetcher == null ? void 0 : fetcher.submit) != null ? _a : actionSubmit;
  const data = (_b = fetcher == null ? void 0 : fetcher.data) != null ? _b : actionData;
  const methods = useForm(formProps);
  const navigation = useNavigation();
  const isSubmittingForm = navigation.state !== "idle" || fetcher && fetcher.state !== "idle";
  const onSubmit = (data2) => {
    setIsSubmittedSuccessfully(true);
    const formData = createFormData(
      { ...data2, ...submitData },
      stringifyAllValues
    );
    submit(formData, {
      method: "post",
      ...submitConfig
    });
  };
  const values = methods.getValues();
  const validKeys = Object.keys(values);
  const onInvalid = () => {
  };
  const formState = methods.formState;
  const {
    dirtyFields,
    isDirty,
    isSubmitSuccessful,
    isSubmitted,
    isSubmitting,
    isValid,
    isValidating,
    touchedFields,
    submitCount,
    errors,
    isLoading,
    disabled
  } = formState;
  const formErrors = mergeErrors(
    errors,
    (data == null ? void 0 : data.errors) ? data.errors : data,
    validKeys
  );
  return {
    ...methods,
    handleSubmit: methods.handleSubmit(
      (_c = submitHandlers == null ? void 0 : submitHandlers.onValid) != null ? _c : onSubmit,
      (_d = submitHandlers == null ? void 0 : submitHandlers.onInvalid) != null ? _d : onInvalid
    ),
    reset: (values2) => {
      setIsSubmittedSuccessfully(false);
      methods.reset(values2);
    },
    register: (name, options) => {
      var _a2, _b2;
      return {
        ...methods.register(name, options),
        ...!(options == null ? void 0 : options.disableProgressiveEnhancement) && {
          defaultValue: (_b2 = (_a2 = data == null ? void 0 : data.defaultValues) == null ? void 0 : _a2[name]) != null ? _b2 : ""
        }
      };
    },
    formState: {
      disabled,
      dirtyFields,
      isDirty,
      isSubmitSuccessful: isSubmittedSuccessfully || isSubmitSuccessful,
      isSubmitted,
      isSubmitting: isSubmittingForm || isSubmitting,
      isValid,
      isValidating,
      touchedFields,
      submitCount,
      isLoading,
      errors: formErrors
    }
  };
};
var RemixFormProvider = ({
  children,
  ...props
}) => {
  return /* @__PURE__ */ React.createElement(FormProvider, { ...props }, children);
};
var useRemixFormContext = () => {
  const methods = useFormContext();
  return {
    ...methods,
    handleSubmit: methods.handleSubmit
  };
};
export {
  RemixFormProvider,
  createFormData,
  getFormDataFromSearchParams,
  getValidatedFormData,
  parseFormData,
  useRemixForm,
  useRemixFormContext,
  validateFormData
};
